/*
 * Tic Tac Toe, ein Spiel fuer zwei Personen.
 * Zwei Spieler setzen auf einem 3x3 Brett abwechselnd
 * Spielfiguren (X,O).
 * Gewonnen hat der Spieler, der zuerst eine Zeile,
 * eine Spalte oder eine der beiden Diagonalen besetzt
 * hat. Geuebte Spieler erreichen mindestens ein Patt.
 */
# include <stdio.h>
# include "matrix.h"

# define BREITE 3

/*
 * Erzeugt leeres Brett.
 * Parameter: Spielbrett, Brettbreite
 */
void initBrett( double **, int);

/*
 * Spieler X (1) oder O (2) setzt.
 * Parameter: Spielbrett, Brettbreite, Spieler
 * Rueckgabe: 1 .. Spieler hat gewonnen,
 *            2 .. Pattstellung, 0 .. sonst
 */
int setze( double **, int, int);

/*
 * Aktuelles Spielbrett.
 * Parameter: Spielbrett, Brettbreite
 */
void zeigeBrett( double **, int);

/*
 * Wertet Spielfeld aus.
 * Parameter: Spielbrett, Brettbreite
 * Rueckgabe: 1 .. Spieler hat gewonnen,
 *            2 .. Pattstellung, 0 .. sonst
 */
int auswerten( double **, int);


/*
 * Startet TicTacToe.
 */
int main()
{
  double **brett = NULL;
  char jn[ 2] = "n";
  int spieler = 1, runden, fertig;

/*
 * Erzeugt leeres Brett als Matrix unter Verwendung
 * der Funktionen aus matrix.h.
 */
  brett = initMatrix( BREITE, BREITE);

  do
  {
    runden = 0;
    initBrett( brett, BREITE);
    zeigeBrett( brett, BREITE);      /* Testausdruck */

    do
    {
      runden++;
      spieler = 3 - spieler;

      fertig = setze( brett, BREITE, spieler);
      zeigeBrett( brett, BREITE);

    } while( !fertig);

    printf( "Fertig! ");
    if( fertig == 1)
    {
      printf( "Sieger nach %d Runden: ", runden);
      printf( "Spieler ");
      if( spieler == 1) printf( "%s\n", "X");
      if( spieler == 2) printf( "%s\n", "O");
    }
    else
      printf( "Patt, Kein Sieger!\n");

    printf( "Noch ein Spiel (j/n)? ");
    scanf( "%1s", jn);
  } while( jn[ 0] == 'j');

  freeMatrix( brett);
  return 0;
}

/*
 * Erzeugt leeres Brett.
 */
void initBrett( double **brett, int n)
{
  int i, j;

  for( i = 0; i < n; i++)                 /* Leeren */
    for( j = 0; j < n; j++)
      brett[ i][ j] = 0;
}

/*
 * Aktuelles Spielbrett.
 */
void zeigeBrett( double **brett, int n)
{
  int i, j;

  for( j = 0; j < n; j++) printf( "\t%d", j);
  printf( "\n");
  for( j = 0; j < n; j++) printf( "\t_");
  printf( "\n\n");

  for( i = 0; i < n; i++)
  {
    printf( "%d | ", i);
    for( j = 0; j < n; j++)
    {
      if( brett[ i][ j] == 0) printf( "\t");
      if( brett[ i][ j] == 1) printf( "\tX");
      if( brett[ i][ j] == 2) printf( "\tO");
    }
    printf( "\n\n");
  }
}

/*
 * Spieler X (1) oder O (2) setzt.
 */
int setze( double **brett, int n, int spieler)
{
  int zeile = 0, spalte = 0;

  printf( "\n");

/* Eingabe */
  do
  {
    printf( "Zug Spieler ");
    if( spieler == 1) printf( "%s\n", "X");
    if( spieler == 2) printf( "%s\n", "O");

    do
    {
      printf( "\tZeile (0 .. %d):  ", n - 1);
      scanf( "%d", &zeile);
    } while( zeile < 0 || zeile >= n);
    do
    {
      printf( "\tSpalte(0 .. %d):  ", n - 1);
      scanf( "%d", &spalte);
    } while( spalte < 0 || spalte >= n);
  } while( brett[ zeile][ spalte] != 0);

/* Setzen */
  brett[ zeile][ spalte] = spieler;

  return auswerten( brett, n);
}


/*
 * Wertet Spielfeld aus.
 */
int auswerten( double **brett, int n)
{
  int i, j;

/* Zeile */
  for( i = 0; i < n; i++)
    if( brett[ i][ 0] != 0 &&
        brett[ i][ 0] == brett[ i][ 1] &&
        brett[ i][ 1] == brett[ i][ 2]) return 1;

/* Spalte */
  for( i = 0; i < n; i++)
    if( brett[ 0][ i] != 0 &&
        brett[ 0][ i] == brett[ 1][ i] &&
        brett[ 1][ i] == brett[ 2][ i]) return 1;

/* Diagonalen */
    if( brett[ 0][ 0] != 0 &&
        brett[ 0][ 0] == brett[ 1][ 1] &&
        brett[ 1][ 1] == brett[ 2][ 2]) return 1;
    if( brett[ 2][ 0] != 0 &&
        brett[ 2][ 0] == brett[ 1][ 1] &&
        brett[ 1][ 1] == brett[ 0][ 2]) return 1;

/* kein Patt, es ex. noch leere Felder */
  for( i = 0; i < n; i++)
    for( j = 0; j < n; j++)
      if( brett[ i][ j] == 0) return 0;

/* sonst Patt */
  return 2;
}
